@@ -1,13 +1,14 @@
 #!/usr/pkg/bin/texlua
 --
+--  $Id: luafindfont.lua 735 2023-06-22 19:12:29Z herbert $
 -----------------------------------------------------------------------
 --         FILE:  luafindfont.lua
 --  DESCRIPTION:  search for fonts in the database
 -- REQUIREMENTS:  luatex v.0.80 or later; packages lualibs, xindex-lapp
---       AUTHOR:  Herbert Voß  (C) 2022-09-02
+--       AUTHOR:  Herbert Voß  (C) 2023-06-21
 -----------------------------------------------------------------------
         luafindfont = luafindfont or { }
-      local version = 0.10
+      local version = 0.13
 luafindfont.version = version
 
 --[[
@@ -30,6 +31,8 @@ Report bugs to  hvoss@tug.org
 
 ]]
 
+-- mtxrun --script font --list --name --all --pattern=times
+
 kpse.set_program_name("luatex")
 local f = kpse.find_file("lualibs.lua")
 
@@ -50,12 +53,13 @@ local args_xetex = 0
 local args_max_string = 90
 
 local otfinfo_arg = ""
+local mtxrun = 0
 local fontNo = 0
 
 local i = 1
 while i <= #arg do
   if arg[i] == "-h" or arg[i] == "--help" then
-    print("Version "..version..", Copyright 2021 by Herbert Voß") 
+    print("Version "..version..", Copyright 2021-23 by Herbert Voß") 
     print([[Syntax: luafindfont [options] <font> 
     By default the Lua program 'luafindfont' creates a list of the
     fonts which have in its names the given string.  
@@ -66,10 +70,15 @@ while i <= #arg do
       ,--no-symbolic-names
     -o,--otfinfo (default 0)
     -i,--info (default 0)
+    -I,--Info (default 0)
     -x, --xetex 
     -v, --verbose
+    -V, --version
     -m,--max_string (default 90)
     <font> (string)  ]])
+  elseif arg[i] == "-V" or arg[i] == "--version" then
+    print("version "..version)
+    os.exit()
   elseif arg[i] == "-v" or arg[i] == "--verbose" then
     args_verbose = 1
   elseif (arg[i] == "-n") or (arg[i] == "--nosymbolicnames") or (arg[i] == "--no-symbolic-names") then
@@ -98,6 +107,15 @@ while i <= #arg do
       print("Option -i needs a following fontnumber!")
       args_info = 0
     end
+  elseif arg[i] == "-I" or arg[i] == "--Info" then
+    mtxrun = 1
+    local I_arg = arg[i+1]
+    fontNo = tonumber(I_arg)
+    if not fontNo then
+      print("Option -I needs a following fontnumber!")
+      fontNo = 0
+    end
+    i = i + 1
   elseif arg[i] == "-m" or arg[i] == "--max_string" then
     local string_len = tonumber(arg[i+1])
     if string_len then
@@ -121,12 +139,12 @@ end
 
 if vlevel > 0 then
   print("Parameter:")
-  print("args_verbose = "..args_verbose)
-  print("args_nosymbolicnames = "..tostring(args_nosymbolicnames))
-  print("args_xetex = "..args_xetex)
-  print("otfinfo_arg = "..otfinfo_arg)
-  print("fontNo = "..fontNo)
-  print("args_max_string = "..args_max_string)
+  print("  args_verbose = "..args_verbose)
+  print("  args_nosymbolicnames = "..tostring(args_nosymbolicnames))
+  print("  args_xetex = "..args_xetex)
+  print("  otfinfo_arg = "..otfinfo_arg)
+  print("  fontNo = "..fontNo)
+  print("  args_max_string = "..args_max_string)
 end
   
 if not args_font then
@@ -136,6 +154,7 @@ end
 
 --local otfinfo = args_otfinfo
 local info = args_info
+local Info = args_Info
 local noSymbolicNames = args_nosymbolicnames
 local maxStrLength = args_max_string
 local font_str = args_font:lower():gsub("%s+", ""):split("&")
@@ -305,7 +324,9 @@ table.sort(fontDataMap,
 	    return false 
 	else 
 	    return string.lower(a["basename"]) < string.lower(b["basename"]) 
-	end end)
+	end 
+end)
+
 -- strip duplicates
 local newFontDataMap = {}
 if #fontDataMap > 0 then
@@ -342,6 +363,12 @@ for i, v in ipairs(fontDataMap) do
   end
 end
 
+if #fontList == 0 then
+  print("There are no fonts with the given search name!\n")
+  os.exit()
+end
+
+
 -- print(l_max[1],l_max[2],l_max[3])
 if l_max[3] > maxStrLength then l_max[3] = maxStrLength end
 
@@ -371,11 +398,20 @@ for i, v in ipairs(fontList) do
   if string.len(path) > l_max[3] then
     path = string.sub (path, 1, minChars).."..."..string.sub (path, string.len(path)-maxStrLength+minChars+4)    
   end
-  local exrun = io.popen("kpsewhich "..v["basename"],'r')
-  if string.len(exrun:read('*all')) > 0 then
-    kpsewhich = "1"
-  else
+  if args_xetex > 0 then
     kpsewhich = "0"
+    local exrun = io.popen("kpsewhich "..v["basename"],'r')
+    if exrun then
+      if string.len(exrun:read('*all')) > 0 then
+        kpsewhich = "1"
+      end
+    else
+      print("!!! There maybe a problem with font "..v["basename"].." kpsewhich doesn't work")
+      print(tostring(exrun))
+	  kpsewhich = "0"
+	  os.exit()
+    end
+    exrun:close()
   end
   if (font_str ~= "*") and not noSymbolicNames then
     if args_xetex > 0 then
@@ -392,47 +428,68 @@ for i, v in ipairs(fontList) do
   end
 end
 
-if fontNo > 0 then
+if fontNo > 0 and mtxrun == 0 then
   print()
-  print("Running otfinfo -"..otfinfo_arg.." on font no."..fontNo)
-  local font = fontList[fontNo]["fullpath"]
-  print("otfinfo -"..otfinfo_arg.." \""..font.."\"")
-  local exrun = io.popen("otfinfo -"..otfinfo_arg.." \""..font.."\"", 'r') -- ".." font may have spaces
-  local output = exrun:read('*all')
-  print(output)
-  exrun:close()
+  if fontNo > #fontList then
+  	print("given font number is greater than the number of the fontlist!")
+  else
+    print("Running otfinfo -"..otfinfo_arg.." on font no."..fontNo)
+    local font = fontList[fontNo]["fullpath"]
+    print("otfinfo -"..otfinfo_arg.." \""..font.."\"")
+    local exrun = io.popen("otfinfo -"..otfinfo_arg.." \""..font.."\"", 'r') -- ".." font may have spaces
+    local output = exrun:read('*all')
+    print(output)
+    exrun:close()
+  end
 end
 
 if info > 0 then
-  font = fontList[info]["familyname"]
-  print("\nFont: "..font)
-  local font_dir = {"local","system","texmf"}
-  local font_ext = {"ttf","otf","ttc"}
-  for j = 1,#font_dir do
-    for i = 1,#font_ext do
-      local fonttype = ""
-      local ext = font_ext[i]
-      local dir = font_dir[j]
-      if fontData["families"][dir][ext]  then  -- font extension exists?
-        local entry = fontData["families"][dir][ext][font]
-        if entry then
-          if entry["r"]  then fonttype = fonttype.."Regular "      end
-          if entry["b"]  then fonttype = fonttype.."| Bold "       end
-          if entry["i"]  then fonttype = fonttype.."| Italic "     end
-          if entry["bi"] then fonttype = fonttype.."| BoldItalic"  end
-          io.write("Fonttype "..ext.."("..dir..") --> ")
-          if #fonttype > 0 then
-            print("| "..fonttype.." |")
+  if info > #fontList then
+  	print("given font number is greater than the number of the fontlist!")
+  else
+    font = fontList[info]["familyname"]
+    print("\nFont: "..font)
+    local font_dir = {"local","system","texmf"}
+    local font_ext = {"ttf","otf","ttc"}
+    for j = 1,#font_dir do
+      for i = 1,#font_ext do
+        local fonttype = ""
+        local ext = font_ext[i]
+        local dir = font_dir[j]
+        if fontData["families"][dir][ext]  then  -- font extension exists?
+          local entry = fontData["families"][dir][ext][font]
+          if entry then
+            if entry["r"]  then fonttype = fonttype.."Regular "      end
+            if entry["b"]  then fonttype = fonttype.."| Bold "       end
+            if entry["i"]  then fonttype = fonttype.."| Italic "     end
+            if entry["bi"] then fonttype = fonttype.."| BoldItalic"  end
+            io.write("Fonttype "..ext.."("..dir..") --> ")
+            if #fonttype > 0 then
+              print("| "..fonttype.." |")
+            else
+              print(" undefined ") --- no regular definiert
+            end
           else
-            print(" undefined ") --- no regular definiert
+            print()
           end
-        else
-          print()
         end
       end
     end
   end
 end
+
+if mtxrun > 0 then
+  print()
+  print("Running mtxrun on font no."..fontNo)
+--  local font = fontList[fontNo]["fullpath"]
+  local font = fontList[fontNo]["basename"]
+  print("mtxrun --script fonts --list --info --file \""..font.."\"")
+  local exrun = io.popen("mtxrun --script fonts --list --info --file \""..font.."\"", 'r') -- ".." font may have spaces
+  local output = exrun:read('*all')
+  print(output)
+  exrun:close()
+end
+
 --print(require 'xindex-pretty'.dump(fontData["families"]["system"]["otf"])) --["families"]["system"]["otf"]))
 
 
